--[[
 Quest companion support module for MiniPet
 Version 5.1.250
 $Revision: 183 $

 All credit for authoring this addon is listed below
]]--

--[[
Copyright (c) 2010-2011, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet quest companion Module" )

local LibLordFarlander = LibStub( "LibLordFarlander-2.0", true )
if( not LibLordFarlander ) then
    return
end--if

if( not LibStub( "LibLordFarlander-Randomizer-2.0", true ) ) then
    return
end--if

local moduleName = "Quest"

local Quest = {
    displayname = L["Quest"],
    settings = {
        [L["forceQuestPet"]] = {
            type = "toggle",
            name = L["Force to pick a quest companion if availiable"],
            width = "double",
            arg = {
                DescriptionString = L["Toggles whether companions that are part of a quest are preferred instead of a random companion."],
                VariableName = "ForceQuestPet",
                Default = true,
            },
            desc = MiniPet.BooleanOptionDescription,
            get = "GetGenericOption",
            set = "SetOption_ResetPets",
        },
    },
    events = {
        "GetCurrentCompanion",
        "ScanPlayer",
        "UNIT_QUEST_LOG_CHANGED",
        "LibQuixote-2.0:Quest_Complete",
        "LibQuixote-2.0:Quest_Gained",
        "LibQuixote-2.0:Quest_Failed",
        "LibQuixote-2.0:Quest_Lost",
    },
    -- Module variables
    questCompanionQuest = nil,
    questCompanion = nil,
}

function Quest:OnScanPlayer( MiniPet, dirtyBags )
    if( self.questCompanionQuest and dirtyBags[self.questCompanionQuest.bag] ) then
        self.questCompanionQuest = nil
    end--if
end--Quest:OnScanPlayer( MiniPet )

function Quest:OnCheckItem( MiniPet, itemID, bag, slot )
    if( MiniPet:GetSetting( moduleName, "profile", "ForceQuestPet" ) ) then
        local questID = PetAndMountDatabase:ItemInSet( itemID, "companion.Quest" )

        if( questID ) then
            local conditions = PetAndMountDatabase.GetConditions( holiday )

            if( conditions and conditions.Quest ) then
                local questIds = TR:GetTableWithInserts( strsplit( "|", conditions.Quest ) )

                for _, questID in ipairs( questIds ) do
                    if( ( questID and self.HasQuestAndNotComplete( tonumber( questID ) ) ) or
                            ( ( self.questCompanion == nil ) and PetAndMountDatabase:ItemInSet( itemID, "companion.Children" ) ) ) then
                        self.questCompanion = ItemList.BuildBagItem( bag, slot )
                        if( questID ) then
                            self.questCompanionQuest = tonumber( questID )
                            break
                        else
                            self.questCompanionQuest = true
                            break
                        end--if
                    end--if
                end--for
                TR:ReturnTable( questIds )
            end--if
            TR:ReturnTable( conditions )
        end--if
    end--if
end--Quest:OnCheckItem( MiniPet, itemID, bag, slot )

function Quest:OnGetCurrentCompanion( MiniPet )
    if( MiniPet:GetSetting( moduleName, "profile", "ForceQuestPet" ) and self.questCompanion and self.questCompanionQuest ) then
        MiniPet:DoCustomEvent( "SetCurrentCompanion", self.questCompanion )
        return self.questCompanion
    end--if
    return nil
end--Quest:OnGetCurrentCompanion( MiniPet )

function Quest:OnQuestDone( MiniPet, _, _, uid )
    if( MiniPet:GetSetting( moduleName, "profile", "ForceQuestPet" ) and ( type( self.questCompanionQuest ) == "number" ) and ( self.questCompanionQuest == uid ) ) then
        self.questCompanionQuest = nil
        self.questCompanion = nil
        MiniPet:SetButton( MiniPet:GetStateBasedOnModifiers() )
    end--if
end--Quest:OnQuestDone( MiniPet, _, _, uid )

function Quest:OnQuest_Gained( MiniPet, _, _, uid )
    if( MiniPet:GetSetting( moduleName, "profile", "ForceQuestPet" ) and ( type( self.questCompanionQuest ) == "number" ) and ( self.questCompanionQuest == uid ) ) then
        MiniPet:DirtyBag()
    end--if
end--Quest:OnQuest_Gained( MiniPet, _, _, uid )

function Quest:OnQuest_Complete( MiniPet, ... )
    self:OnQuestDone( MiniPet, ... )
end--Quest:OnQuest_Complete( MiniPet, ... )

function Quest:OnQuest_Failed( MiniPet, ... )
    self:OnQuestDone( MiniPet, ... )
end--Quest:OnQuest_Failed( MiniPet, ... )

function Quest:OnQuest_Lost( MiniPet, ... )
    self:OnQuestDone( MiniPet, ... )
end--Quest:OnQuest_Lost( MiniPet, ... )

function Quest:OnUNIT_QUEST_LOG_CHANGED( MiniPet, _, unit )
    if( not LibStub( "LibQuixote-2.0", true ) ) then
        if( ( unit == "player" ) and MiniPet:GetSetting( moduleName, "profile", "ForceQuestPet" ) and ( type( self.questCompanionQuest ) == "number" ) and self.questCompanionQuest ) then
            if( self.HasQuestAndNotComplete( self.questCompanionQuest ) ) then
                if( not self.questCompanion ) then
                    MiniPet:DirtyBag()
                end--if
            else
                self.questCompanionQuest = nil
                self.questCompanion = nil
                MiniPet:SetButton( MiniPet:GetStateBasedOnModifiers() )
            end--if
        end--if
    end--if
end--Quest:OnUNIT_QUEST_LOG_CHANGED( MiniPet, _, unit )

function Quest.HasQuestAndNotComplete( questID )
    if( _G["Baldrick"] ) then
        return _G["Baldrick"].HasQuestAndNotComplete( questID )
    else
        local Quixote = LibStub( "LibQuixote-2.0", true )

        if( Quixote ) then
            local uid, _, _, _, _, _, complete, _, _, _ = Quixote:GetQuestByUid( questID )

            return ( uid ~= nil) and ( not complete )
        else
            local numEntries, _ = GetNumQuestLogEntries()

            if( numEntries > 0 ) then
                for id = 1, numEntries do
                    local complete = select( 6, GetQuestLogTitle( id ) )

                    if( ( not complete ) and ( LibLordFarlander.GetItemIDFromLink( GetQuestLink( id ) ) == questID ) ) then
                        return true
                    end--if
                end--for
            end--if
        end--if
    end--if
    return false
end--Quest.HasQuestAndNotComplete( questID )

LibStub( "LibLordFarlander-Randomizer-2.0" ):RegisterModuleToHandler( "MiniPet", moduleName, Quest, "Companions" )
